#========================================================================================
# package      DE_EPAGES::Shop::t::unit::UrlHandler
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UrlHandler.pm
#========================================================================================
package DE_EPAGES::Shop::t::unit::UrlHandler;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Dictionary::API::Template;
use DE_EPAGES::Shop::API::TLE::UrlHandler;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant LANGUAGE_ID    => 1;
use constant LANGUAGE_CODE2 => 'de';

sub test {
    my $self = shift;

    $self->test_getHelpCenterUrl();
    $self->test_getHelpCenterUrlUnity();
    done_testing();

    return;
}

sub test_getHelpCenterUrl {
    my $self = shift;

    my $UrlHandler    = $self->_createUrlHandler();
    my $System        = $self->_createSystem();
    my $User          = $self->_createUser(0);
    my $HelpCenterUrl = $UrlHandler->getHelpCenterUrl( $System, $User );
    my $ExpectedHelpCenterUrl =
        'https://help.epages.com/base/'
      . LANGUAGE_CODE2;

    is( $HelpCenterUrl, $ExpectedHelpCenterUrl, 'getHelpCenterLink' );

    return;
}

sub test_getHelpCenterUrlUnity {
    my $self = shift;

    my $UrlHandler    = $self->_createUrlHandler();
    my $System        = $self->_createSystem();
    my $User          = $self->_createUser(1);
    my $HelpCenterUrl = $UrlHandler->getHelpCenterUrl( $System, $User );
    my $ExpectedHelpCenterUrl =
        'https://help.epages.com/now/'
      . LANGUAGE_CODE2;

    is( $HelpCenterUrl, $ExpectedHelpCenterUrl, 'getUnityHelpCenterLink' );

    return;
}

sub _createUrlHandler {
    my $self = shift;

    my $UrlHandler       = DE_EPAGES::Shop::API::TLE::UrlHandler->new();
    my $MockedUrlHandler = Test::MockObject::Extends->new($UrlHandler);

    $MockedUrlHandler->mock(
        '_getInfoLanguage',
        sub {
            my $self = shift;
            my ($LanguageID) = @_;

            if ( $LanguageID == LANGUAGE_ID ) {
                return { 'Code2' => LANGUAGE_CODE2 };
            }
            else {
                Error('INVALID_LANG_ID', {'LangID' => $LanguageID});
            }
        }
    );

    $MockedUrlHandler->mock(
        '_getTemplate',
        sub {
            my $innerSelf = shift;

            return $self->_createTemplate();
        }
    );

    return $MockedUrlHandler;
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'HelpCenterUrl'       => 'https://help.epages.com/base/<LanguageCode2>',
        'HelpCenterUrlUnity'  => 'https://help.epages.com/now/<LanguageCode2>',
    };
    my $System = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Site', $hAttributes );

    return $System;
}

sub _createUser {
    my $self = shift;
    my ($IsUnityRegistered) = @_;

    my $hAttributes = { 'LanguageID' => LANGUAGE_ID,
        'Site' => DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop',
            { 'IsUnityRegistered' => $IsUnityRegistered }
        )
    };
    my $User = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAttributes );

    return $User;
}

sub _createTemplate {
    my $self = shift;

    my $Template = DE_EPAGES::Dictionary::API::Template->new();
    my $MockedTemplate = Test::MockObject::Extends->new($Template);

    $MockedTemplate->mock(
        'dictionary',
        sub {
            my $self = shift;
            my ($FileName, $LanguageCode2) = @_;

            my $SupportedLangCode2 = LANGUAGE_CODE2;

            if ( $LanguageCode2 eq $SupportedLangCode2 ) {
                return {
                    $SupportedLangCode2 => {
                        'HelpLanguageID' => $SupportedLangCode2
                    }
                };
            }
            else {
                Error('INVALID_LANG_CODE2', {'Code2' => $LanguageCode2});
            }

        }
    );

    return $MockedTemplate;
}

__PACKAGE__->runScript();
