#==============================================================================
# package      DE_EPAGES::ShoppingList::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to save and delete objects.
#========================================================================================
package DE_EPAGES::ShoppingList::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use DE_EPAGES::Database::API::Connection       qw( GetCurrentDBHandle );
use DE_EPAGES::ShoppingList::API::ShoppingList qw( AddProductToShoppingList GetUnityShoppingList );

use JSON;
use MIME::Base64 qw(encode_base64);

#==============================================================================
# Event handlers
#==============================================================================
sub JSONAddItemToShoppingList {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();
    my $Form = $Servlet->form;
    my $User = $Servlet->user;

    my $ProductGUID = $Form->data()->{'ProductGUID'};
    my $Quantity = $Form->data()->{'Quantity'};
    my $LineItemGUID = $Form->data()->{'LineItemGUID'};
    my $LocaleID = $Form->data()->{'LocaleID'} // $Servlet->locale;
    return unless defined $ProductGUID;
    return unless defined $Quantity;


    if( defined $User and (not $User->get('IsAnonymous')) ) {
        # get the shopping list from user and add products
        my $ShoppingList = GetUnityShoppingList($User);

        my $Product = $Shop->loadObjectByGuid($ProductGUID, 'Product');
        my $hProductParams = {'Quantity' => $Quantity};

        # use then LineItemGUID parameter if we add an item from the BASKET to the list
        # otherwise the quantity is not changed
        $hProductParams->{'LineItemGUID'} = $LineItemGUID if $LineItemGUID;
        AddProductToShoppingList($ShoppingList, $Product, $Shop, $hProductParams);

        my $hParams = {
                        'success'   => JSON::true,
                        'wishlist'  => $User->get('WebUrlSSL') . '&ViewAction=UnityViewUserShoppingList&Locale=' . $LocaleID,
                        'productId' => $ProductGUID
                      };

        $self->writeContentOutput($hParams);

    } else {

        #create a login url with the changeaction for adding products
        my $Location = $Shop->get('WebServerScriptUrlSSL') . "/$LocaleID/" ;

        my @Params;
        push @Params, 'ViewAction=UnityViewRegistrationRedirect';
        my $LastViewActionParameters = 'ProductGUID=' . $ProductGUID . '&Quantity=' . $Quantity;
        $LastViewActionParameters .= '&LineItemGUID=' . $LineItemGUID if defined $LineItemGUID;
        push @Params, 'LastViewActionParameters=' . encode_base64($LastViewActionParameters,'');
        push @Params, 'LastViewAction=UnityViewShoppingList';

        $Location .= '?' . join('&', @Params) if scalar(@Params) > 0;

        my $hResult = {'success' => JSON::false,
                       'reason'  => 'nouser',
                       'redirect' => $Location,
                       'productId' => $ProductGUID
                      };

        $self->writeContentOutput($hResult);

    }
}

sub JSONDeleteItemFromShoppingList {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;

    my $ShoppingList = $Servlet->object;
    my $ItemID       = $Form->value('ItemID');
    my $Shop         = $ShoppingList->getSite();

    my $ShoppingListEntry = $Shop->loadObject($ItemID, 'ShoppingListItem');

    $ShoppingListEntry->delete();

    return $self->writeContentOutput({'success' => JSON::true  });
}

sub JSONUpdateShoppingListItemQuantity {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;

    my $ShoppingList = $Servlet->object;
    my $ItemID       = $Form->value('ItemID');
    my $Quantity     = $Form->value('Quantity');
    my $Shop         = $ShoppingList->getSite();

    my $ShoppingListEntry = $Shop->loadObject($ItemID, 'ShoppingListItem');
    my $Product = $ShoppingListEntry->get('Product');

    if ( defined $Product) {
        $ShoppingListEntry->set({'Quantity' => $Quantity});
    }

    #set shoppinglist change date
    $ShoppingList->set( { 'ChangeDate' => GetCurrentDBHandle()->currentDateTime() });

    return $self->writeContentOutput({'success' => JSON::true  });
}

1;
